@@ -1,8 +1,15 @@
+"""Provides the TemplatedScriptTask.
+
+Copyright 2021, Bill & Melinda Gates Foundation. All rights reserved.
+"""
+import copy
+import inspect
 import os
+from contextlib import suppress
 from dataclasses import dataclass, field
 from functools import partial
 from logging import getLogger, DEBUG
-from typing import List, Callable, Type, Dict, Any, Union
+from typing import List, Callable, Type, Dict, Any, Union, TYPE_CHECKING
 from jinja2 import Environment
 from idmtools.assets import AssetCollection, Asset
 from idmtools.entities import CommandLine
@@ -10,25 +17,55 @@ from idmtools.entities.itask import ITask
 from idmtools.entities.iworkflow_item import IWorkflowItem
 from idmtools.entities.simulation import Simulation
 from idmtools.registry.task_specification import TaskSpecification
+if TYPE_CHECKING:  # pragma: no cover
+    from idmtools.entities.iplatform import IPlatform
 
 logger = getLogger(__name__)
-WINDOWS_DEFAULT_WRAPPER = """
-set PYTHONPATH=%cd%\\Assets\\;%PYTHONPATH%
-%*
+
+LINUX_DICT_TO_ENVIRONMENT = """{% for key, value in vars.items() %}
+export {{key}}="{{value}}"
+{% endfor %}
 """
-LINUX_DEFAULT_WRAPPER = """
-export PYTHONPATH=$(pwd)/Assets/:$PYTHONPATH
-%*
+
+WINDOWS_DICT_TO_ENVIRONMENT = """{% for key, value in vars.items() %}
+set {{key}}="{{value}}"
+{% endfor %}
 """
 
+# Define our common windows templates
+WINDOWS_BASE_WRAPPER = """echo Running %*
+%*"""
+WINDOWS_PYTHON_PATH_WRAPPER = """set PYTHONPATH=%cd%\\Assets\\site-packages\\;%cd%\\Assets\\;%PYTHONPATH%
+{}""".format(WINDOWS_BASE_WRAPPER)
+WINDOWS_DICT_TO_ENVIRONMENT = WINDOWS_DICT_TO_ENVIRONMENT + WINDOWS_BASE_WRAPPER
+
+# Define our linux common scripts
+LINUX_BASE_WRAPPER = """echo Running $@
+\"$@\"
+"""
+LINUX_PYTHON_PATH_WRAPPER = """export PYTHONPATH=$(pwd)/Assets/site-packages:$(pwd)/Assets/:$PYTHONPATH
+{}""".format(LINUX_BASE_WRAPPER)
+LINUX_DICT_TO_ENVIRONMENT = LINUX_DICT_TO_ENVIRONMENT + LINUX_BASE_WRAPPER
+
 
 @dataclass()
 class TemplatedScriptTask(ITask):
     """
-    Defines a task to run a script using a template. Best suited to shell scripts
+    Defines a task to run a script using a template. Best suited to shell scripts.
+
+    Examples:
+        In this example, we add modify the Python Path using TemplatedScriptTask and LINUX_PYTHON_PATH_WRAPPER
+
+        .. literalinclude:: ../../examples/cookbook/python/python-path/python-path.py
+
+        In this example, we modify environment variable using TemplatedScriptTask and LINUX_DICT_TO_ENVIRONMENT
+
+        .. literalinclude:: ../../examples/cookbook/environment/variables/environment-vars.py
     """
     #: Name of script
     script_path: str = field(default=None, metadata={"md": True})
+    #: If platform requires path to script executing binary(ie /bin/bash)
+    script_binary: str = field(default=None, metadata={"md": True})
     #: The template contents
     template: str = field(default=None, metadata={"md": True})
     #: The template file. You can only use either template or template_file at once
@@ -49,6 +86,7 @@ class TemplatedScriptTask(ITask):
     gather_transient_asset_hooks: List[Callable[[ITask], AssetCollection]] = field(default_factory=list)
 
     def __post_init__(self):
+        """Constructor."""
         super().__post_init__()
         if self.script_path is None and self.template_file is None:
             raise ValueError("Either script name or template file is required")
@@ -84,7 +122,8 @@ class TemplatedScriptTask(ITask):
     def _add_template_to_asset_collection(task: 'TemplatedScriptTask', asset_collection: AssetCollection) -> \
             AssetCollection:
         """
-        Add our rendered template to the asset collection
+        Add our rendered template to the asset collection.
+
         Args:
             task: Task to add
             asset_collection:Asset collection
@@ -113,7 +152,7 @@ class TemplatedScriptTask(ITask):
 
     def gather_common_assets(self) -> AssetCollection:
         """
-        Gather common(experiment-level) assets for task
+        Gather common(experiment-level) assets for task.
 
         Returns:
             AssetCollection containing common assets
@@ -126,7 +165,7 @@ class TemplatedScriptTask(ITask):
 
     def gather_transient_assets(self) -> AssetCollection:
         """
-        Gather transient(experiment-level) assets for task
+        Gather transient(experiment-level) assets for task.
 
         Returns:
             AssetCollection containing transient assets
@@ -141,19 +180,19 @@ class TemplatedScriptTask(ITask):
 
     def reload_from_simulation(self, simulation: Simulation):
         """
-        Reload a templated script task. When reloading, you will only have the rendered template available
+        Reload a templated script task. When reloading, you will only have the rendered template available.
 
         Args:
             simulation:
 
         Returns:
-
+            None
         """
         # check experiment level assets for our script
         if simulation.parent.assets:
             # prep new asset collection in case we have to remove our asset from the experiment level
             new_assets = AssetCollection()
-            for i, asset in enumerate(simulation.parent.assets.assets):
+            for _i, asset in enumerate(simulation.parent.assets.assets):
                 # is it our script?
                 if asset.filename != self.script_path and asset.absolute_path != self.script_path:
                     # nope keep it
@@ -161,35 +200,45 @@ class TemplatedScriptTask(ITask):
             # set filtered assets back to parent
             simulation.parent.assets = new_assets
 
-    def pre_creation(self, parent: Union[Simulation, IWorkflowItem]):
+    def pre_creation(self, parent: Union[Simulation, IWorkflowItem], platform: 'IPlatform'):
         """
-        Before creating simulation, we need to set our command line
+        Before creating simulation, we need to set our command line.
 
         Args:
             parent: Parent object
-
+            platform: Platform item is being ran on
         Returns:
 
         """
         # are we experiment or simulation level asset?
-        if self.template_is_common:
-            sn = f'Assets{self.path_sep}{self.script_path}'
+        if self.script_binary:
+            sn = self.script_binary + ' '
         else:
-            sn = self.script_path
+            sn = ''
+        if self.template_is_common:
+            sn += platform.join_path(platform.common_asset_path, self.script_path)
+        else:
+            sn += self.script_path
         # set the command line to the rendered script
-        self.command = CommandLine(sn)
+        self.command = CommandLine.from_string(sn)
+        if self.path_sep != "/":
+            self.command.executable = self.command.executable.replace("/", self.path_sep)
+            self.command.is_windows = True
         # set any extra arguments
         if self.extra_command_arguments:
-            self.command.add_argument(self.extra_command_arguments)
-        # run base precreation
-        super().pre_creation(parent)
+            other_command = CommandLine.from_string(self.extra_command_arguments)
+            self.command._args.append(other_command.executable)
+            if other_command._options:
+                self.command._args += other_command._options
+            if other_command._args:
+                self.command._args += other_command._args
+        super().pre_creation(parent, platform)
 
 
 @dataclass()
 class ScriptWrapperTask(ITask):
     """
-    Allows you to wrap a script with another script
-
+    Allows you to wrap a script with another script.
 
     See Also:
         :py:class:`idmtools_models.templated_script_task.TemplatedScriptTask`
@@ -201,17 +250,69 @@ class ScriptWrapperTask(ITask):
     task: ITask = field(default=None)
 
     def __post_init__(self):
+        """Constructor."""
         if self.template_script_task is None:
             raise ValueError("Template Script Task is required")
 
         if self.task is None:
             raise ValueError("Task is required")
 
+        if isinstance(self.task, dict):
+            self.task = self.from_dict(self.task)
+        if isinstance(self.template_script_task, dict):
+            self.template_script_task = self.from_dict(self.template_script_task)
+
+    @staticmethod
+    def from_dict(task_dictionary: Dict[str, Any]):
+        """Load the task from a dictionary."""
+        from idmtools.core.task_factory import TaskFactory
+        task_args = {k: v for k, v in task_dictionary.items() if k not in ['task_type']}
+        return TaskFactory().create(task_dictionary['task_type'], **task_args)
+
+    @property
+    def command(self):
+        """Our task property. Again, we have to overload this because of wrapping a task."""
+        cmd = copy.deepcopy(self.template_script_task.command)
+        cmd.add_raw_argument(str(self.task.command))
+        return cmd
+
+    @command.setter
+    def command(self, value: Union[str, CommandLine]):
+        """Set our command. because we are wrapping a task, we have to overload this."""
+        callers = []
+        with suppress(KeyError, IndexError):
+            callers = inspect.stack()[1:3]
+        if not callers or callers[0].filename == __file__ or callers[1].filename == __file__:
+            if isinstance(value, property):
+                self._command = None
+            elif isinstance(value, str):
+                self._command = CommandLine.from_string(value)
+            else:
+                self._command = value
+        else:
+            self.task.command = value
+
+    @property
+    def wrapped_task(self):
+        """
+        Our task we are wrapping with a shell script.
+
+        Returns:
+            Our wrapped task
+        """
+        return self.task
+
+    @wrapped_task.setter
+    def wrapped_task(self, value: ITask):
+        """Set our wrapped task."""
+        return None if isinstance(value, property) else value
+
     def gather_common_assets(self):
         """
-        Gather all the common assets
-        Returns:
+        Gather all the common assets.
 
+        Returns:
+            Common assets(Experiment Assets)
         """
         self.common_assets.add_assets(self.template_script_task.gather_common_assets())
         self.common_assets.add_assets(self.task.gather_common_assets())
@@ -219,9 +320,10 @@ class ScriptWrapperTask(ITask):
 
     def gather_transient_assets(self) -> AssetCollection:
         """
-        Gather all the transient assets
-        Returns:
+        Gather all the transient assets.
 
+        Returns:
+            Transient Assets(Simulation level assets)
         """
         self.transient_assets.add_assets(self.template_script_task.gather_transient_assets())
         self.transient_assets.add_assets(self.task.gather_transient_assets())
@@ -229,47 +331,49 @@ class ScriptWrapperTask(ITask):
 
     def reload_from_simulation(self, simulation: Simulation):
         """
-        Reload from simulation
+        Reload from simulation.
 
         Args:
             simulation: simulation
 
         Returns:
-
+            None
         """
-        from idmtools.core.task_factory import TaskFactory
-        # check if the task is a dict
-        if isinstance(simulation.task.task, dict):
-            # process sub task first
-            task_args = {k: v for k, v in simulation.task.task.items() if k not in ['task_type']}
-            simulation.task.task = TaskFactory().create(simulation.task.task['task_type'], **task_args)
-            simulation.task.task.reload_from_simulation(simulation)
+        pass
 
-            simulation.task.template_script_task = simulation.task.task
-        else:
-            logger.warning("Unable to load subtask")
-
-    def pre_creation(self, parent: Union[Simulation, IWorkflowItem]):
+    def pre_creation(self, parent: Union[Simulation, IWorkflowItem], platform: 'IPlatform'):
         """
-        Before creation, create the true command by adding the wrapper name
+        Before creation, create the true command by adding the wrapper name.
 
+        Here we call both our wrapped task and our template_script_task pre_creation
         Args:
-            parent:
+            parent: Parent Task
+            platform: Platform Templated Task is executing on
 
         Returns:
-
+            None
         """
-        self.task.pre_creation(parent)
-        # get command from wrapper command and add to wrapper script as item we call as argument to script
-        self.template_script_task.extra_command_arguments = str(self.task.command)
-        self.template_script_task.pre_creation(parent)
-        self.command = self.template_script_task.command
+        self.task.pre_creation(parent, platform)
+        self.template_script_task.pre_creation(parent, platform)
 
-    def post_creation(self, parent: Union[Simulation, IWorkflowItem]):
-        self.task.post_creation(parent)
-        self.template_script_task.post_creation(parent)
+    def post_creation(self, parent: Union[Simulation, IWorkflowItem], platform: 'IPlatform'):
+        """
+        Post creation of task.
+
+        Here we call both our wrapped task and our template_script_task post_creation
+
+        Args:
+            parent: Parent of task
+            platform: Platform we are running on
+
+        Returns:
+            None
+        """
+        self.task.post_creation(parent, platform)
+        self.template_script_task.post_creation(parent, platform)
 
     def __getattr__(self, item):
+        """Proxy get attr to child except for task item and items not in our object."""
         if item not in self.__dict__:
             return getattr(self.task, item)
         else:
@@ -280,7 +384,7 @@ def get_script_wrapper_task(task: ITask, wrapper_script_name: str, template_cont
                             template_file: str = None, template_is_common: bool = True,
                             variables: Dict[str, Any] = None, path_sep: str = '/') -> ScriptWrapperTask:
     """
-    Convenience function that will wrap a task for you with some defaults
+    Convenience function that will wrap a task for you with some defaults.
 
     Args:
         task: Task to wrap
@@ -295,6 +399,7 @@ def get_script_wrapper_task(task: ITask, wrapper_script_name: str, template_cont
         ScriptWrapperTask wrapping the task
 
     See Also:
+        :class:`idmtools_models.templated_script_task.TemplatedScriptTask`
         :func:`idmtools_models.templated_script_task.get_script_wrapper_windows_task`
         :func:`idmtools_models.templated_script_task.get_script_wrapper_unix_task`
     """
@@ -312,18 +417,13 @@ def get_script_wrapper_task(task: ITask, wrapper_script_name: str, template_cont
 
 
 def get_script_wrapper_windows_task(task: ITask, wrapper_script_name: str = 'wrapper.bat',
-                                    template_content: str = WINDOWS_DEFAULT_WRAPPER,
+                                    template_content: str = WINDOWS_DICT_TO_ENVIRONMENT,
                                     template_file: str = None, template_is_common: bool = True,
                                     variables: Dict[str, Any] = None) -> ScriptWrapperTask:
     """
-    Get wrapper script task for windows platforms
+    Get wrapper script task for windows platforms.
 
-    The default content wraps a bash script that adds the assets directory to the python path
-
-    .. code-block:: batch
-
-        set PYTHONPATH=%cd%/Assets/;%PYTHONPATH%
-        %*
+    The default content wraps a another task with a batch script that exports the variables to the run environment defined in variables. To modify python path, use WINDOWS_PYTHON_PATH_WRAPPER
 
     You can adapt this script to modify any pre-scripts you need or call others scripts in succession
 
@@ -339,6 +439,7 @@ def get_script_wrapper_windows_task(task: ITask, wrapper_script_name: str = 'wra
         ScriptWrapperTask
 
     See Also::
+        :class:`idmtools_models.templated_script_task.TemplatedScriptTask`
         :func:`idmtools_models.templated_script_task.get_script_wrapper_task`
         :func:`idmtools_models.templated_script_task.get_script_wrapper_unix_task`
     """
@@ -346,44 +447,44 @@ def get_script_wrapper_windows_task(task: ITask, wrapper_script_name: str = 'wra
                                    variables, "\\")
 
 
-def get_script_wrapper_unix_task(task: ITask, wrapper_script_name: str = 'wrapper.sh', template_content: str = None,
+def get_script_wrapper_unix_task(task: ITask, wrapper_script_name: str = 'wrapper.sh', template_content: str = LINUX_DICT_TO_ENVIRONMENT,
                                  template_file: str = None, template_is_common: bool = True,
                                  variables: Dict[str, Any] = None):
     """
-        Get wrapper script task for unix platforms
+    Get wrapper script task for unix platforms.
 
-        The default content wraps a bash script that adds the assets directory to the python path
+    The default content wraps a another task with a bash script that exports the variables to the run environment defined in variables. To modify python path, you can use LINUX_PYTHON_PATH_WRAPPER
 
-        .. code-block:: bash
+    You can adapt this script to modify any pre-scripts you need or call others scripts in succession
 
-            set PYTHONPATH=$(pwd)/Assets/:$PYTHONPATH
-            %*
+    Args:
+        task: Task to wrap
+        wrapper_script_name: Wrapper script name(defaults to wrapper.sh)
+        template_content:  Template Content
+        template_file: Template File
+        template_is_common: Is the template experiment level
+        variables: Variables for template
 
-        You can adapt this script to modify any pre-scripts you need or call others scripts in succession
+    Returns:
+        ScriptWrapperTask
 
-        Args:
-            task: Task to wrap
-            wrapper_script_name: Wrapper script name(defaults to wrapper.sh)
-            template_content:  Template Content
-            template_file: Template File
-            template_is_common: Is the template experiment level
-            variables: Variables for template
-
-        Returns:
-            ScriptWrapperTask
-
-        See Also:
-        :func:`idmtools_models.templated_script_task.get_script_wrapper_task`
-        :func:`idmtools_models.templated_script_task.get_script_wrapper_windows_task`
-        """
+    See Also:
+    :class:`idmtools_models.templated_script_task.TemplatedScriptTask`
+    :func:`idmtools_models.templated_script_task.get_script_wrapper_task`
+    :func:`idmtools_models.templated_script_task.get_script_wrapper_windows_task`
+    """
     return get_script_wrapper_task(task, wrapper_script_name, template_content, template_file, template_is_common,
                                    variables, "/")
 
 
 class TemplatedScriptTaskSpecification(TaskSpecification):
+    """
+    TemplatedScriptTaskSpecification provides the plugin specs for TemplatedScriptTask.
+    """
+
     def get(self, configuration: dict) -> TemplatedScriptTask:
         """
-        Get instance of TemplatedScriptTask with configuration
+        Get instance of TemplatedScriptTask with configuration.
 
         Args:
             configuration: configuration for TemplatedScriptTask
@@ -395,7 +496,7 @@ class TemplatedScriptTaskSpecification(TaskSpecification):
 
     def get_description(self) -> str:
         """
-        Get description of plugin
+        Get description of plugin.
 
         Returns:
             Plugin description
@@ -404,7 +505,7 @@ class TemplatedScriptTaskSpecification(TaskSpecification):
 
     def get_example_urls(self) -> List[str]:
         """
-        Get example urls related to TemplatedScriptTask
+        Get example urls related to TemplatedScriptTask.
 
         Returns:
             List of urls that have examples related to CommandTask
@@ -413,18 +514,32 @@ class TemplatedScriptTaskSpecification(TaskSpecification):
 
     def get_type(self) -> Type[TemplatedScriptTask]:
         """
-        Get task type provided by plugin
+        Get task type provided by plugin.
 
         Returns:
             TemplatedScriptTask
         """
         return TemplatedScriptTask
 
+    def get_version(self) -> str:
+        """
+        Returns the version of the plugin.
+
+        Returns:
+            Plugin Version
+        """
+        from idmtools_models import __version__
+        return __version__
+
 
 class ScriptWrapperTaskSpecification(TaskSpecification):
+    """
+    ScriptWrapperTaskSpecification defines the plugin specs for ScriptWrapperTask.
+    """
+
     def get(self, configuration: dict) -> ScriptWrapperTask:
         """
-        Get instance of ScriptWrapperTask with configuration
+        Get instance of ScriptWrapperTask with configuration.
 
         Args:
             configuration: configuration for ScriptWrapperTask
@@ -436,7 +551,7 @@ class ScriptWrapperTaskSpecification(TaskSpecification):
 
     def get_description(self) -> str:
         """
-        Get description of plugin
+        Get description of plugin.
 
         Returns:
             Plugin description
@@ -445,7 +560,7 @@ class ScriptWrapperTaskSpecification(TaskSpecification):
 
     def get_example_urls(self) -> List[str]:
         """
-        Get example urls related to ScriptWrapperTask
+        Get example urls related to ScriptWrapperTask.
 
         Returns:
             List of urls that have examples related to CommandTask
@@ -454,9 +569,19 @@ class ScriptWrapperTaskSpecification(TaskSpecification):
 
     def get_type(self) -> Type[ScriptWrapperTask]:
         """
-        Get task type provided by plugin
+        Get task type provided by plugin.
 
         Returns:
             TemplatedScriptTask
         """
         return ScriptWrapperTask
+
+    def get_version(self) -> str:
+        """
+        Returns the version of the plugin.
+
+        Returns:
+            Plugin Version
+        """
+        from idmtools_models import __version__
+        return __version__
